@@ -25,13 +25,16 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include <glib/gi18n.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+
 #include "typedefs.h"
 #include "bookmarks.h"
 #include "catalog.h"
 #include "file-utils.h"
 #include "preferences.h"
 
-
+#define MAX_LINE_LENGTH 4096
 #define MAX_LINES 100
 
 
@@ -94,57 +97,6 @@ bookmarks_free (Bookmarks *bookmarks)
 }
 
 
-
-char *
-bookmarks_utils__get_menu_item_name (const char *path)
-{
-	char     *tmp_path;
-	gboolean  catalog_or_search;
-	char     *name;
-
-	tmp_path = g_strdup (pref_util_remove_prefix (path));
-
-	/* if it is a catalog then remove the extension */
-	
-	catalog_or_search = (pref_util_location_is_catalog (path)
-			     || pref_util_location_is_search (path));
-	
-	if (catalog_or_search)
-		tmp_path[strlen (tmp_path) - strlen (CATALOG_EXT)] = 0;
-			 
-	if (strcmp (tmp_path, "/") == 0) 
-		name = g_strdup ("/");
-	else {
-		if (catalog_or_search) {
-			char *base_path;
-			int   l;
-
-			base_path = get_catalog_full_path (NULL);
-			l = strlen (base_path);
-			g_free (base_path);
-
-			name = g_strdup (tmp_path + 1 + l);
-		} else {
-			const char *base_path;
-			int         l;
-			
-			base_path = g_get_home_dir ();
-			l = strlen (base_path);
-			
-			if ((strcmp (tmp_path, base_path) != 0)
-			    && (strncmp (tmp_path, base_path, l) == 0))
-				name = g_strdup (tmp_path + 1 + l);
-			else
-				name = g_strdup (tmp_path);
-		}
-	}
-	
-	g_free (tmp_path);
-	
-	return name;
-}
-
-
 static char *
 get_menu_item_tip (const char *path)
 {	
@@ -154,8 +106,7 @@ get_menu_item_tip (const char *path)
 
 	tmp_path = g_strdup (path);
 
-	if (pref_util_location_is_catalog (tmp_path)
-	    || pref_util_location_is_search (tmp_path)) {
+	if (uri_scheme_is_catalog (tmp_path) || uri_scheme_is_search (tmp_path)) {
 		gchar *rc_dir_prefix;
 
 		/* if it is a catalog then remove the extension */
@@ -172,7 +123,7 @@ get_menu_item_tip (const char *path)
 		g_free (rc_dir_prefix);
 	}
 
-	tip = g_strdup (pref_util_remove_prefix (tmp_path) + offset);
+	tip = g_strdup (remove_scheme_from_uri (tmp_path) + offset);
 	g_free (tmp_path);
 
 	return tip;
@@ -235,7 +186,7 @@ bookmarks_add (Bookmarks   *bookmarks,
 
 	my_insert (bookmarks->names, 
 		   path, 
-		   bookmarks_utils__get_menu_item_name (path));
+		   get_uri_display_name (path));
 
 	my_insert (bookmarks->tips, 
 		   path, 
@@ -271,8 +222,8 @@ bookmarks_remove (Bookmarks  *bookmarks,
 		return;
 
 	bookmarks->list = g_list_remove_link (bookmarks->list, link);
-	g_list_free (link);
 	g_free (link->data);
+	g_list_free (link);
 
 	if (get_link_from_path (bookmarks->list, path) == NULL) {
 		my_remove (bookmarks->names, path);
@@ -339,13 +290,13 @@ bookmarks_remove_all (Bookmarks *bookmarks)
 }
 
 
-#define MAX_LINE_LENGTH 4096
 void
 bookmarks_load_from_disk (Bookmarks *bookmarks)
 {
-	gchar  line [MAX_LINE_LENGTH];
-	gchar *path;
-	FILE  *f;
+	GnomeVFSResult  result;
+	GnomeVFSHandle *handle;
+	char           *uri;
+	char            line [MAX_LINE_LENGTH];
 
 	g_return_if_fail (bookmarks != NULL);
 
@@ -353,85 +304,87 @@ bookmarks_load_from_disk (Bookmarks *bookmarks)
 	if (bookmarks->rc_filename == NULL)
 		return;
 
-	path = g_strconcat (g_get_home_dir (),
-			    "/",
-			    bookmarks->rc_filename,
-			    NULL);
+	uri = g_strconcat (get_home_uri (),
+			   "/",
+			   bookmarks->rc_filename,
+			   NULL);
+	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
+	g_free (uri);
 
-	f = fopen (path, "r");
-	g_free (path);
+	if (result != GNOME_VFS_OK)
+		return;
 
-	if (!f)	return;
-
-	while (fgets (line, sizeof (line), f)) {
+	while (_gnome_vfs_read_line (handle,
+				     line,
+				     MAX_LINE_LENGTH,
+				     NULL) == GNOME_VFS_OK) {
 		char *path;
 
 		if (line[0] != '"')
 			continue;
 
-		line[strlen (line) - 2] = 0;
+		line[strlen (line) - 1] = 0;
 		path = line + 1;
 
 		bookmarks->list = g_list_prepend (bookmarks->list, 
 						  g_strdup (path));
 		my_insert (bookmarks->names, 
 			   path, 
-			   bookmarks_utils__get_menu_item_name (path));
+			   get_uri_display_name (path));
 
 		my_insert (bookmarks->tips, 
 			   path, 
 			   get_menu_item_tip (path));
 	}
-	fclose (f);
+
+	gnome_vfs_close (handle);
 
 	bookmarks->list = g_list_reverse (bookmarks->list);
 }
-#undef MAX_LINE_LENGTH	
 
 
 void
 bookmarks_write_to_disk (Bookmarks *bookmarks)
 {
-	FILE *f;
-	gchar *path;
-	GList *scan;
-	gint lines;
+	GnomeVFSResult  result;
+	GnomeVFSHandle *handle;
+	char           *uri;
+	int             lines;
+	GList          *scan;
 
 	g_return_if_fail (bookmarks != NULL);
+
 	if (bookmarks->rc_filename == NULL)
 		return;
 
-	path = g_strconcat (g_get_home_dir (),
-			    "/",
-			    bookmarks->rc_filename,
-			    NULL);
+	uri = g_strconcat (get_home_uri (),
+			   "/",
+			   bookmarks->rc_filename,
+			   NULL);
+	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_WRITE);
+	g_free (uri);
 
-	f = fopen (path, "w+");
-	g_free (path);
-	
-	if (!f)	{
-		g_print ("ERROR opening bookmark file\n");
+	if (result != GNOME_VFS_OK)
 		return;
-	}
-	
+
 	/* write the file list. */
+
 	lines = 0;
 	scan = bookmarks->list; 
 	while (((bookmarks->max_lines < 0) || (lines < bookmarks->max_lines)) 
-	       && scan) {
-		if (! fprintf (f, "\"%s\"\n", (gchar*) scan->data)) {
+	       && (scan != NULL)) {
+		if (_gnome_vfs_write_line (handle,
+					   "\"%s\"",
+					   (char*) scan->data) != GNOME_VFS_OK) {
 			g_print ("ERROR saving to bookmark file\n");
-			fclose (f);
-			
-			return;
+			break;
 		}
 		lines++;
 		scan = scan->next;
 	}
 
-	fclose (f);
+	gnome_vfs_close (handle);
 }
-#undef lines
 
 
 const gchar *
