@@ -22,25 +22,27 @@
 
 #include <config.h>
 #include <string.h>
+#include <time.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
+#include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-dateedit.h>
 #include <glade/glade.h>
 
 #include "typedefs.h"
+#include "preferences.h"
 #include "main.h"
 #include "glib-utils.h"
-#include "gthumb-window.h"
+#include "gth-window.h"
 #include "gtk-utils.h"
 #include "gth-file-view.h"
 #include "comments.h"
 #include "gth-exif-utils.h"
 #include "dlg-comment.h"
 #include "dlg-image-prop.h"
+#include "file-utils.h"
 
 
 enum {
@@ -55,10 +57,12 @@ enum {
 };
 
 
+#define DIALOG_NAME "comment"
 #define GLADE_FILE "gthumb_comments.glade"
 
+
 typedef struct {
-	GThumbWindow  *window;
+	GthWindow     *window;
 	GladeXML      *gui;
 	GList         *file_list;
 
@@ -99,24 +103,19 @@ static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	data->window->comment_dlg = NULL;
+	gth_window_set_comment_dlg (GTH_WINDOW (data->window), NULL);
 	g_object_unref (data->gui);
 	free_dialog_data (data);
 	g_free (data);
 }
 
 
-static int
-strcmp_null_tollerant (const char *s1, const char *s2)
+static gboolean
+unrealize_cb (GtkWidget  *widget, 
+	  DialogData *data)
 {
-	if ((s1 == NULL) && (s2 == NULL))
-		return 0;
-	else if ((s1 != NULL) && (s2 == NULL))
-		return 1;
-	else if ((s1 == NULL) && (s2 != NULL))
-		return -1;
-	else 
-		return strcmp (s1, s2);
+	pref_util_save_window_geometry (GTK_WINDOW (widget), DIALOG_NAME);
+	return FALSE;
 }
 
 
@@ -177,6 +176,7 @@ save_clicked_cb (GtkWidget  *widget,
 	struct tm    tm = {0};
 	GtkTextIter  start_iter, end_iter;
 	gboolean     save_changed;
+	gboolean     close_on_save;
 
 	save_changed = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->save_changed_checkbutton));
 	cdata = comment_data_new ();
@@ -261,13 +261,19 @@ save_clicked_cb (GtkWidget  *widget,
 			comments_save_comment_non_null (filename, cdata);
 		else
 			comments_save_comment (filename, cdata);
-		all_windows_notify_update_comment (filename); 
+		all_windows_notify_update_metadata (filename); 
 	}
 
-	if ((data->window != NULL) && (data->window->image_prop_dlg != NULL))
-		dlg_image_prop_update (data->window->image_prop_dlg);
-
+	gth_window_update_current_image_metadata (data->window);
 	comment_data_free (cdata);
+
+	/**/
+
+	g_object_get (data->window,
+		      "close_on_save", &close_on_save,
+		      NULL);
+	if (close_on_save)
+		dlg_comment_close (data->dialog);
 }
 
 
@@ -373,15 +379,17 @@ get_exif_date_option_item (DialogData *data)
 
 
 GtkWidget *
-dlg_comment_new (GThumbWindow *window)
+dlg_comment_new (GthWindow *window)
 {
-	DialogData   *data;
-	GtkWidget    *btn_close;
-	GtkWidget    *btn_help;
-
-	if (window->comment_dlg != NULL) {
-		gtk_window_present (GTK_WINDOW (window->comment_dlg));
-		return window->comment_dlg;
+	GtkWidget  *current_dialog;
+	DialogData *data;
+	GtkWidget  *btn_close;
+	GtkWidget  *btn_help;
+	
+	current_dialog = gth_window_get_comment_dlg (window);
+	if (current_dialog != NULL) {
+		gtk_window_present (GTK_WINDOW (current_dialog));
+		return current_dialog;
 	}
 
 	data = g_new0 (DialogData, 1);
@@ -396,7 +404,8 @@ dlg_comment_new (GThumbWindow *window)
 
 	/* Get the widgets. */
 
-	window->comment_dlg = data->dialog = glade_xml_get_widget (data->gui, "comments_dialog");
+	data->dialog = glade_xml_get_widget (data->gui, "comments_dialog");
+	gth_window_set_comment_dlg (window, data->dialog);
 	data->comment_main_box = glade_xml_get_widget (data->gui, "comment_main_box");
 	data->note_text_view = glade_xml_get_widget (data->gui, "note_text");
 	data->place_entry = glade_xml_get_widget (data->gui, "place_entry");
@@ -417,6 +426,10 @@ dlg_comment_new (GThumbWindow *window)
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
+	g_signal_connect (G_OBJECT (data->dialog), 
+			  "unrealize",
+			  G_CALLBACK (unrealize_cb),
+			  data);
 	g_signal_connect (G_OBJECT (data->save_button), 
 			  "clicked",
 			  G_CALLBACK (save_clicked_cb),
@@ -447,17 +460,11 @@ dlg_comment_new (GThumbWindow *window)
 	/* run dialog. */
 
 	gtk_widget_grab_focus (data->note_text_view);
-
-	{
-		GtkWidget *parent_win = window->viewer;
-		while (parent_win != NULL && !GTK_IS_WINDOW(parent_win))
-			parent_win = parent_win->parent;
-		if (parent_win != NULL)
-			gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (parent_win));
-	}
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
+				      GTK_WINDOW (window));
 
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
-	gtk_widget_show (data->dialog);
+	pref_util_restore_window_geometry (GTK_WINDOW (data->dialog), DIALOG_NAME);
 	dlg_comment_update (data->dialog);
 
 	return data->dialog;
@@ -481,7 +488,7 @@ dlg_comment_update (GtkWidget *dlg)
 	
 	free_dialog_data (data);
 
-	data->file_list = gth_file_view_get_file_list_selection (data->window->file_list->view);
+	data->file_list = gth_window_get_file_list_selection (data->window); 
 
 	if (data->file_list == NULL) {
 		gtk_widget_set_sensitive (data->save_button, FALSE);
@@ -513,12 +520,12 @@ dlg_comment_update (GtkWidget *dlg)
 #else
 	{
 		char *first_image = data->file_list->data;
-		data->have_exif_data = have_exif_data (first_image);
+		data->have_exif_data = have_exif_time (first_image);
 	}
 #endif
 	
 	first_image = data->file_list->data;
-	data->original_cdata = cdata = comments_load_comment (first_image);
+	data->original_cdata = cdata = comments_load_comment (first_image, TRUE);
 
 	if (cdata != NULL) {
 		comment_data_free_keywords (cdata);
@@ -527,7 +534,7 @@ dlg_comment_update (GtkWidget *dlg)
 		for (scan = data->file_list->next; scan; scan = scan->next) {
 			CommentData *scan_cdata;
 
-			scan_cdata = comments_load_comment (scan->data);
+			scan_cdata = comments_load_comment (scan->data, TRUE);
 
 			/* If there is no comment then all fields must be
 			 * considered to differ. */
