@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -28,21 +28,22 @@
 #include <string.h>
 #include <time.h>
 
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-dialog.h>
-#include <libgnomeui/gnome-dialog-util.h>
-#include <libgnomeui/gnome-file-entry.h>
 #include <libgnomeui/gnome-icon-theme.h>
 #include <libgnomeui/gnome-icon-lookup.h>
+#include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <gphoto2/gphoto2-context.h>
 #include <gphoto2/gphoto2-camera.h>
 #include <gphoto2/gphoto2-abilities-list.h>
 #include <glade/glade.h>
 
 #include "gtk-utils.h"
-#include "gthumb-window.h"
+#include "gth-window.h"
+#include "gth-browser.h"
 #include "file-data.h"
 #include "file-utils.h"
 #include "dlg-categories.h"
@@ -52,9 +53,9 @@
 #include "gconf-utils.h"
 #include "preferences.h"
 #include "pixbuf-utils.h"
+#include "rotation-utils.h"
 #include "main.h"
 
-
 #define GLADE_FILE "gthumb_camera.glade"
 #define CATEGORIES_GLADE_FILE "gthumb_comment.glade"
 #define CAMERA_FILE "gphoto-48.png"
@@ -73,7 +74,7 @@ typedef enum {
 
 
 typedef struct {
-	GThumbWindow   *window;
+	GthBrowser     *browser;
 	GladeXML       *gui;
 
 	GtkWidget      *dialog;
@@ -81,8 +82,7 @@ typedef struct {
 	GtkWidget      *import_preview_scrolledwindow;
 	GtkWidget      *camera_model_label; 
 	GtkWidget      *select_model_button;
-	GtkWidget      *destination_fileentry;
-	GtkWidget      *destination_entry;
+	GtkWidget      *destination_filechooserbutton;
 	GtkWidget      *film_entry;
 	GtkWidget      *keep_names_checkbutton;
 	GtkWidget      *delete_checkbutton;
@@ -93,6 +93,7 @@ typedef struct {
 	GtkWidget      *import_preview_box;
 	GtkWidget      *import_reload_button;
 	GtkWidget      *import_delete_button;
+	GtkWidget      *import_ok_button;
 	GtkWidget      *i_commands_table;
 
 	GtkWidget      *progress_info_image;
@@ -113,6 +114,7 @@ typedef struct {
 
 	gboolean             keep_original_filename;
 	gboolean             delete_from_camera;
+	gboolean             adjust_orientation;
 
 	int                  image_n;
 	char                *local_folder;
@@ -130,6 +132,7 @@ typedef struct {
 
 	GList               *categories_list;
 	GList               *delete_list;
+	GList               *adjust_orientation_list;
 	GList               *saved_images_list;
 
 	/**/
@@ -140,6 +143,7 @@ typedef struct {
 	gboolean             thread_done; 
 
 	guint                idle_id;
+	
 } DialogData;
 
 
@@ -148,8 +152,8 @@ static void
 destroy_cb (GtkWidget  *widget, 
 	    DialogData *data)
 {
-	GThumbWindow *window = data->window;
-	gboolean      thread_done;
+	GthBrowser *browser = data->browser;
+	gboolean    thread_done;
 
 	/* Remove check. */
 
@@ -177,7 +181,7 @@ destroy_cb (GtkWidget  *widget,
 	/**/
 
 	if (data->view_folder)
-		window_go_to_directory (data->window, data->local_folder);
+		gth_browser_go_to_directory (data->browser, data->local_folder);
 
 	/**/
 
@@ -191,6 +195,7 @@ destroy_cb (GtkWidget  *widget,
 		g_object_unref (data->camera_present_pixbuf);
 	path_list_free (data->categories_list);
 	path_list_free (data->delete_list);
+	path_list_free (data->adjust_orientation_list);
 	path_list_free (data->saved_images_list);
 	gp_camera_unref (data->camera);
 	gp_context_unref (data->context);
@@ -203,7 +208,7 @@ destroy_cb (GtkWidget  *widget,
 
 	if (ImportPhotos) {
 		ImportPhotos = FALSE;
-		window_close (window);
+		gth_window_close (GTH_WINDOW (browser));
 	}
 }
 
@@ -500,6 +505,7 @@ main_dialog_set_sensitive (DialogData *data,
 			   gboolean    value)
 {
 	gtk_widget_set_sensitive (data->select_model_button, value);
+	gtk_widget_set_sensitive (data->import_ok_button, value);
 	gtk_widget_set_sensitive (data->import_reload_button, value);
 	gtk_widget_set_sensitive (data->import_delete_button, value);
 	gtk_widget_set_sensitive (data->i_commands_table, value);
@@ -681,7 +687,7 @@ get_mime_type_from_filename (const char*filename)
 {
 	const char *result;
 
-	char *n1 = g_filename_to_utf8 (filename, -1, 0, 0, 0);
+	char *n1 = g_filename_display_name (filename);
 	char *n2 = g_utf8_strdown (n1, -1);
 	char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
 	result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
@@ -731,6 +737,7 @@ load_images_preview__step (AsyncOperationData *aodata,
 			    data->context);
 	
 	tmp_filename = get_temp_filename ();
+
 	if (gp_file_save (file, tmp_filename) >= 0) {
 		GdkPixbuf *pixbuf;
 		int        width, height;
@@ -762,9 +769,11 @@ load_images_preview__step (AsyncOperationData *aodata,
 						 NULL,
 						 fdata);
 		g_object_unref (pixbuf);
-		unlink (tmp_filename);
+		file_unlink (tmp_filename);
 	}
+
 	g_free (tmp_filename);
+	g_free (camera_folder);
 	
 	gp_file_unref (file);
 }
@@ -1037,16 +1046,18 @@ is_valid_filename (const char *name)
 static char*
 get_folder_name (DialogData *data)
 {
-	char *entry_val;
+	char *esc_destination;
 	char *destination;
 	char *film_name;
 	char *path;
 
-	entry_val = _gtk_entry_get_filename_text (GTK_ENTRY (data->destination_entry));
-	destination = remove_ending_separator (entry_val);
-	g_free (entry_val);
+	esc_destination = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (data->destination_filechooserbutton));
+	destination = gnome_vfs_unescape_string (esc_destination, "");
+	g_free (esc_destination);
 
-	if (! dlg_check_folder (data->window, destination)) {
+	eel_gconf_set_path (PREF_PHOTO_IMPORT_DESTINATION, destination);
+
+	if (! dlg_check_folder (GTH_WINDOW (data->browser), destination)) {
 		g_free (destination);
 		return NULL;
 	}
@@ -1069,8 +1080,6 @@ get_folder_name (DialogData *data)
 
 	path = g_build_filename (destination, film_name, NULL);
 	g_free (film_name);
-
-	eel_gconf_set_path (PREF_PHOTO_IMPORT_DESTINATION, destination);
 	g_free (destination);
 
 	return path;
@@ -1156,7 +1165,7 @@ add_categories_to_image (DialogData *data,
 	if (data->categories_list == NULL)
 		return;
 
-	cdata = comments_load_comment (filename);
+	cdata = comments_load_comment (filename, FALSE);
 	if (cdata == NULL)
 		cdata = comment_data_new ();
 	
@@ -1191,13 +1200,15 @@ save_image (DialogData *data,
 			    GP_FILE_TYPE_NORMAL,
 			    file, 
 			    data->context);
-	g_free (camera_folder);
 
 	local_path = get_file_name (data, camera_path, local_folder, n);
 
 	if (gp_file_save (file, local_path) >= 0) {
 		if (data->delete_from_camera) 
 			data->delete_list = g_list_prepend (data->delete_list, g_strdup (camera_path));
+		if (data->adjust_orientation) {
+			data->adjust_orientation_list = g_list_prepend (data->adjust_orientation_list, g_strdup (local_path));
+		}
 		data->saved_images_list = g_list_prepend (data->saved_images_list, g_strdup (camera_path));
 		add_categories_to_image (data, local_path);
 	} else {
@@ -1206,7 +1217,8 @@ save_image (DialogData *data,
 		data->interrupted = TRUE;
 		g_mutex_unlock (data->yes_or_no);
 	}
-	
+
+	g_free (camera_folder);	
 	g_free (local_path);
 	gp_file_unref (file);
 }
@@ -1217,7 +1229,7 @@ add_film_keyword (const char *folder)
 {
 	CommentData *cdata;
 
-	cdata = comments_load_comment (folder);
+	cdata = comments_load_comment (folder, FALSE);
 	if (cdata == NULL)
 		cdata = comment_data_new ();
 	comment_data_add_keyword (cdata, _("Film"));
@@ -1250,8 +1262,7 @@ static void
 delete_images__done (AsyncOperationData *aodata, 
 		     DialogData         *data)
 {
-	GThumbWindow *window = data->window;
-	gboolean      interrupted;
+	gboolean interrupted;
 
 	task_terminated (data);
 
@@ -1264,39 +1275,12 @@ delete_images__done (AsyncOperationData *aodata,
 
 	data->view_folder = TRUE;
 
-	if (ImportPhotos) {
-		ImportPhotos = FALSE;
-		gtk_widget_show (window->app);
-	}
+        if (ImportPhotos) {
+                ImportPhotos = FALSE;
+                gtk_widget_show (GTK_WIDGET (data->browser));
+        }
 
-	gtk_widget_destroy (data->dialog);
-}
-
-
-static void 
-save_images__init (AsyncOperationData *aodata, 
-		   DialogData         *data)
-{
-	all_windows_remove_monitor ();
-
-	data->image_n = 1;
-	if (data->delete_list != NULL) {
-		path_list_free (data->delete_list);
-		data->delete_list = NULL;
-	}
-	if (data->saved_images_list != NULL) {
-		path_list_free (data->saved_images_list);
-		data->saved_images_list = NULL;
-	}
-}
-
-
-static void 
-save_images__step (AsyncOperationData *aodata, 
-		   DialogData         *data)
-{
-	const char *camera_path = aodata->scan->data;
-	save_image (data, camera_path, data->local_folder, data->image_n++);
+        gtk_widget_destroy (data->dialog);
 }
 
 
@@ -1320,6 +1304,90 @@ notify_cb (gpointer cb_data)
 }
 
 
+static void 
+adjust_orientation__done (AsyncOperationData *aodata,
+			  DialogData         *data)
+{
+	gboolean            interrupted;
+	AsyncOperationData *new_aodata;
+
+	g_mutex_lock (data->yes_or_no);
+	interrupted = data->interrupted;
+	g_mutex_unlock (data->yes_or_no);
+
+	data->idle_id = g_idle_add (notify_cb, data);
+
+	if (interrupted)
+		return;
+
+	new_aodata = async_operation_new (data->delete_list,
+					  NULL,
+					  delete_images__step,
+					  delete_images__done,
+					  data);
+	async_operation_start (new_aodata);
+}
+
+
+static void 
+adjust_orientation__step (AsyncOperationData *aodata,
+			  DialogData         *data)
+{
+	const char       *filepath = aodata->scan->data;
+	GnomeVFSFileInfo  info;
+	GtkWindow        *window = GTK_WINDOW (data->dialog);
+	
+	gnome_vfs_get_file_info (filepath, &info, GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS|GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	
+	if (file_is_image (filepath, TRUE)) {
+		FileData     *fd = file_data_new (filepath, &info);
+		RotationData *rot_data = rotation_data_new ();
+
+		update_rotation_from_exif_data (fd->path, rot_data);
+		if (image_is_jpeg (filepath))
+			apply_transformation_jpeg (window, fd->path, rot_data);
+		else
+			apply_transformation_generic (window, fd->path, rot_data);
+
+		file_data_unref (fd);
+		g_free (rot_data);
+	}
+	
+	gnome_vfs_set_file_info (filepath, &info, GNOME_VFS_SET_FILE_INFO_PERMISSIONS|GNOME_VFS_SET_FILE_INFO_OWNER);
+}
+
+
+static void 
+save_images__init (AsyncOperationData *aodata, 
+		   DialogData         *data)
+{
+	all_windows_remove_monitor ();
+
+	data->image_n = 1;
+	if (data->delete_list != NULL) {
+		path_list_free (data->delete_list);
+		data->delete_list = NULL;
+	}
+	if (data->adjust_orientation_list != NULL) {
+		path_list_free (data->adjust_orientation_list);
+		data->adjust_orientation_list = NULL;
+	}
+	if (data->saved_images_list != NULL) {
+		path_list_free (data->saved_images_list);
+		data->saved_images_list = NULL;
+	}
+}
+
+
+static void 
+save_images__step (AsyncOperationData *aodata, 
+		   DialogData         *data)
+{
+	const char *camera_path = aodata->scan->data;
+	save_image (data, camera_path, data->local_folder, data->image_n++);
+}
+
+
 static void 
 save_images__done (AsyncOperationData *aodata, 
 		   DialogData         *data)
@@ -1333,15 +1401,13 @@ save_images__done (AsyncOperationData *aodata,
 	error = data->error;
 	g_mutex_unlock (data->yes_or_no);
 
-	data->idle_id = g_idle_add (notify_cb, data);
-
 	if (interrupted || error)
 		return;
 
-	new_aodata = async_operation_new (data->delete_list,
+	new_aodata = async_operation_new (data->adjust_orientation_list,
 					  NULL,
-					  delete_images__step,
-					  delete_images__done,
+					  adjust_orientation__step,
+					  adjust_orientation__done,
 					  data);
 	async_operation_start (new_aodata);
 }
@@ -1393,6 +1459,7 @@ ok_clicked_cb (GtkButton  *button,
 
 	data->keep_original_filename = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->keep_names_checkbutton));
 	data->delete_from_camera = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->delete_checkbutton));
+	data->adjust_orientation = eel_gconf_get_boolean (PREF_PHOTO_IMPORT_ADJUST_ORIENTATION, TRUE);
 
 	eel_gconf_set_boolean (PREF_PHOTO_IMPORT_KEEP_FILENAMES, data->keep_original_filename);
 	eel_gconf_set_boolean (PREF_PHOTO_IMPORT_DELETE, data->delete_from_camera);
@@ -1435,7 +1502,7 @@ ok_clicked_cb (GtkButton  *button,
 	if (! ensure_dir_exists (data->local_folder, 0755)) {
 		char *utf8_path;
 		char *msg;
-		utf8_path = g_filename_to_utf8 (data->local_folder, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_display_name (data->local_folder);
 		msg = g_strdup_printf (_("Could not create the folder \"%s\": %s"),
 				       utf8_path,
 				       errno_to_string ());
@@ -1452,7 +1519,7 @@ ok_clicked_cb (GtkButton  *button,
 	if (! check_permissions (data->local_folder, R_OK | W_OK | X_OK)) {
 		char *utf8_path;
 		char *msg;
-		utf8_path = g_filename_to_utf8 (data->local_folder, -1, NULL, NULL, NULL);
+		utf8_path = g_filename_display_name (data->local_folder);
 		msg = g_strdup_printf (_("You don't have the right permissions to create images in the folder \"%s\""), utf8_path);
 
 		display_error_dialog (data, _("Could not import photos"), msg);
@@ -1686,17 +1753,17 @@ start_operation (DialogData    *data,
 
 
 void
-dlg_photo_importer (GThumbWindow *window)
+dlg_photo_importer (GthBrowser *browser)
 {
 	DialogData   *data;
-	GtkWidget    *btn_ok, *btn_cancel;
+	GtkWidget    *btn_cancel;
 	GdkPixbuf    *mute_pixbuf;
 	char         *default_path;
+	char         *esc_uri;
 	char         *default_film_name;
-	GValue        value = {0, };
 
 	data = g_new0 (DialogData, 1);
-	data->window = window;
+	data->browser = browser;
 
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE , NULL, NULL);
         if (!data->gui) {
@@ -1736,8 +1803,7 @@ dlg_photo_importer (GThumbWindow *window)
 	data->import_preview_scrolledwindow = glade_xml_get_widget (data->gui, "import_preview_scrolledwindow");
 	data->camera_model_label = glade_xml_get_widget (data->gui, "camera_model_label");
 	data->select_model_button = glade_xml_get_widget (data->gui, "select_model_button");
-	data->destination_fileentry = glade_xml_get_widget (data->gui, "destination_fileentry");
-	data->destination_entry = glade_xml_get_widget (data->gui, "destination_entry");
+	data->destination_filechooserbutton = glade_xml_get_widget (data->gui, "destination_filechooserbutton");
 	data->film_entry = glade_xml_get_widget (data->gui, "film_entry");
 	data->keep_names_checkbutton = glade_xml_get_widget (data->gui, "keep_names_checkbutton");
 	data->delete_checkbutton = glade_xml_get_widget (data->gui, "delete_checkbutton");
@@ -1752,7 +1818,7 @@ dlg_photo_importer (GThumbWindow *window)
 	data->import_reload_button = glade_xml_get_widget (data->gui, "import_reload_button");
 	data->import_delete_button = glade_xml_get_widget (data->gui, "import_delete_button");
 	data->i_commands_table = glade_xml_get_widget (data->gui, "i_commands_table");
-	btn_ok = glade_xml_get_widget (data->gui, "import_okbutton");
+	data->import_ok_button = glade_xml_get_widget (data->gui, "import_okbutton");
 	btn_cancel = glade_xml_get_widget (data->gui, "import_cancelbutton");
 
 	data->image_list = gth_image_list_new (THUMB_SIZE + THUMB_BORDER);
@@ -1765,14 +1831,6 @@ dlg_photo_importer (GThumbWindow *window)
 
 	/* Set widgets data. */
 
-	/* Make use of the new filechooser */
-
-	g_value_init (&value, G_TYPE_BOOLEAN);
-	g_value_set_boolean (&value, TRUE);
-	g_object_set_property (G_OBJECT (data->destination_fileentry), "use_filechooser", &value);
-
-	/**/
-
 	data->camera_present_pixbuf = gdk_pixbuf_new_from_file (GTHUMB_GLADEDIR "/" CAMERA_FILE, NULL);
 	mute_pixbuf = gdk_pixbuf_new_from_file (GTHUMB_GLADEDIR "/" MUTE_FILE, NULL);
 
@@ -1799,8 +1857,9 @@ dlg_photo_importer (GThumbWindow *window)
 	default_path = eel_gconf_get_path (PREF_PHOTO_IMPORT_DESTINATION, NULL);
 	if ((default_path == NULL) || (*default_path == 0))
 		default_path = g_strdup (g_get_home_dir());
-	gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (data->destination_fileentry), default_path);
-	_gtk_entry_set_filename_text (GTK_ENTRY (data->destination_entry), default_path);
+	esc_uri = gnome_vfs_escape_host_and_path_string (default_path);
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (data->destination_filechooserbutton), esc_uri);
+	g_free (esc_uri);
 	g_free (default_path);
 
 	default_film_name = eel_gconf_get_path (PREF_PHOTO_IMPORT_FILM, "");
@@ -1815,7 +1874,7 @@ dlg_photo_importer (GThumbWindow *window)
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
-	g_signal_connect (G_OBJECT (btn_ok), 
+	g_signal_connect (G_OBJECT (data->import_ok_button), 
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
@@ -1843,7 +1902,7 @@ dlg_photo_importer (GThumbWindow *window)
 
 	/* run dialog. */
 
-	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (window->app));
+	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_widget_show (data->dialog);
 
@@ -2071,3 +2130,4 @@ dlg_select_camera_model_cb (GtkButton  *button,
 
 
 #endif /*HAVE_LIBGPHOTO */
+
