@@ -3,7 +3,7 @@
 /*
  *  GThumb
  *
- *  Copyright (C) 2003 Free Software Foundation, Inc.
+ *  Copyright (C) 2003, 2004, 2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,10 +23,13 @@
 
 #include <config.h>
 #include <string.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+
 #include "file-utils.h"
 #include "glib-utils.h"
 #include "gth-exif-utils.h"
@@ -105,6 +108,7 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 {
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
+	GValue             value = { 0, };
 
 	edv->priv->scrolled_win = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (edv->priv->scrolled_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
@@ -140,6 +144,12 @@ gth_exif_data_viewer_construct (GthExifDataViewer *edv)
 							   renderer,
 							   "text", VALUE_COLUMN,
 							   NULL);
+
+	g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
+	g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
+	g_object_set_property (G_OBJECT (renderer), "ellipsize", &value);
+	g_value_unset (&value);
+
 	gtk_tree_view_append_column (GTK_TREE_VIEW (edv->priv->image_exif_view),
 				     column);
 
@@ -336,22 +346,21 @@ static void
 update_exif_data (GthExifDataViewer *edv,
 		  ExifData          *edata)
 {
+	const char   *path;
 	unsigned int  i;
 	gboolean      date_added = FALSE;
 	gboolean      aperture_added = FALSE;
 	gboolean      last_entry_is_void = ! edv->priv->view_file_info;
 	gboolean      list_is_empty = TRUE;
 
-	if (edv->priv->path == NULL)
+	path = get_file_path_from_uri (edv->priv->path);
+	if (path == NULL)
 		return;
 
 	if (edata == NULL)
-		edata = exif_data_new_from_file (edv->priv->path);
+		edata = exif_data_new_from_file (path);
 	else
 		exif_data_ref (edata);
-	
-	if (edata == NULL) 
-                return;
 
 	if (edata == NULL) 
                 return;
@@ -460,10 +469,9 @@ update_file_info (GthExifDataViewer *edv)
 	if (edv->priv->viewer == NULL)
 		return;
 
-	utf8_name = g_filename_to_utf8 (file_name_from_path (edv->priv->path), -1, 
-					NULL, NULL, NULL);
+	utf8_name = g_filename_display_basename (edv->priv->path);
 
-	if (!image_viewer_is_void(IMAGE_VIEWER (edv->priv->viewer))) {
+	if (!image_viewer_is_void (IMAGE_VIEWER (edv->priv->viewer))) {
 		width = image_viewer_get_image_width (edv->priv->viewer);
 		height = image_viewer_get_image_height (edv->priv->viewer);
 	} else {
@@ -511,6 +519,13 @@ update_file_info (GthExifDataViewer *edv)
 			    POS_COLUMN, -1,
 			    -1);
 
+	gtk_list_store_append (edv->priv->image_exif_model, &iter);
+	gtk_list_store_set (edv->priv->image_exif_model, &iter,
+			    NAME_COLUMN, _("Type"),
+			    VALUE_COLUMN, gnome_vfs_mime_get_description (gnome_vfs_get_file_mime_type (edv->priv->path, NULL, FALSE)),
+			    POS_COLUMN, -1,
+			    -1);
+
 	/**/
 
 	g_free (utf8_time_txt);
