@@ -28,7 +28,7 @@
 #include "pixbuf-utils.h"
 
 
-#define PREVIEW_SIZE 300
+#define PREVIEW_SIZE 350
 #define DRAG_THRESHOLD 1
 #define BORDER 3
 #define BORDER2 (BORDER * 2)
@@ -965,6 +965,8 @@ size_request (GtkWidget      *widget,
 		widget->requisition.height = gdk_pixbuf_get_height (priv->pixbuf);
 	}
 
+	widget->requisition.width = MAX (widget->requisition.width, PREVIEW_SIZE);
+
 	/* Chain up to default that simply reads current requisition */
 	GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 }
@@ -1079,7 +1081,7 @@ init (GthImageSelector *selector)
 GType
 gth_image_selector_get_type ()
 {
-        static guint type = 0;
+        static GType type = 0;
 
         if (! type) {
                 GTypeInfo type_info = {
@@ -1253,8 +1255,8 @@ gth_image_selector_get_selection (GthImageSelector *selector,
 
 	selection->x = MAX (selection->x, 0);
 	selection->y = MAX (selection->y, 0);
-	selection->width = MIN (selection->width, priv->pixbuf_width);
-	selection->height = MIN (selection->height, priv->pixbuf_height);
+	selection->width = MIN (selection->width, priv->pixbuf_width - selection->x);
+	selection->height = MIN (selection->height, priv->pixbuf_height - selection->y);
 }
 
 
